/*
** Soubor: serva_c.c
** Knihovna funkci pro ovladani serv pomoc multiplexace
** Autor: Viktor Obr (xobrvi00@stud.feec.vutbr.cz
*/

#define F_CPU 16000000		        // definice frekvence procesoru
#include <avr\io.h>				    // definicni soubor pro mikrokontroler ATmega16
#include <avr\interrupt.h>		    // definicni soubor pro obsluhu preruseni
#include "serva_h.h"

// konstanty

unsigned char servo_position[8];
unsigned char adr = 0, timer = 255;

/*
inicializace timer0 a serv - position 0-250
timer0 - CTC mode, preddelicka = 64
       - OCR0 = 250 => preteceni za 1 ms
       - povoleni preruseni pri komparaci
PORTB3:0 vystupni - PORTB3 impulzy
                  - PORTB2:0 adresa
pocatecni natoceni serv
*/

void servo_init( unsigned char position)
{
     TCCR0 = (1<<WGM01)|(0<<WGM00)|(0<<CS02)|(1<<CS01)|(1<<CS00);
     OCR0 = 250;
     TIMSK = (1<<OCIE0);
     
     DDRB |= 0b00001111;
     
     int i;
     for( i=0; i<8; i++)
     {
          servo_position[i] = position;
     }
}

// zmena polohy serva - position 0-250, servo 0-7

void servo_position_new( unsigned char position, unsigned char servo)
{
     servo_position[servo] = position;
}

// obsluha preruseni - navysovani promenne timer a akce podle jeji hodnoty

ISR( TIMER0_COMP_vect )
{
     timer++;
     if( timer==0 )
     {
          adr = 0;
		  PORTB = 8;                             // PORTB = 0b00001000
          OCR0 = 250;
     }
     else if( timer==4 || timer==8 || timer==12 || timer==16 || timer==20 || timer==24 || timer==28 )
     {
          adr++;
		  PORTB = 8 + adr;                       // PORTB = 0b00001000 + (1 az 7)
          OCR0 = 250;
     }
     else if( timer==1 || timer==5 || timer==9 || timer==13 || timer==17 || timer==21 || timer==25 || timer==29 )
     {
          OCR0 = servo_position[adr];
     }
     else if( timer==2 || timer==6 || timer==10 || timer==14 || timer==18 || timer==22 || timer==26 || timer==30 )
     {
		  PORTB = 0 + adr;                      // PORTB = 0b00000000 + (0 az 7)
          OCR0 = 250 - servo_position[adr];
     }
     else if( timer==3 || timer==7 || timer==11 || timer==15 || timer==19 || timer==23 || timer==27 )
     {
          OCR0 = 125;
     }
     else if( timer==31 )
     {
          OCR0 = 125;
          timer = 255;
     }
}
